#ifndef RooTestPdf_h
#define RooTestPdf_h

#include <math.h>
#include <iomanip>
#include <iostream> 
#include <fstream>
#include "TComplex.h"
#include "TString.h"
#include "TMath.h"
#include "TRandom3.h"
#include "TFile.h"
#include "TChain.h"
#include "TLorentzVector.h"
#include "TStopwatch.h"
#include "TH1F.h"
#if defined(USEROOT) || defined(__CINT__)
#include "RooStringVar.h"
#include "RooAbsPdf.h"
#include "RooAbsData.h"
#include "RooRealVar.h"
#include "RooAbsReal.h"
#include "RooRealProxy.h"
#include "RooListProxy.h"
#include "RooFitResult.h"
#include "RooArgSet.h"
#include "RooRealConstant.h"
//#include "TSpline.h"
#include "TTree.h"
//#include "TMap.h"
#else
#include "RooAbsPdf.h"
#include "RooRealVar.h"
#include "RooAbsData.h"
#include "RooFitResult.h"
#include "RooRealVar.h"
#include "RooRealProxy.h"
#include "RooListProxy.h"
#include "RooRandom.h"
#include "RooArgSet.h"
#include "RooRealConstant.h"
#endif
#include "LineShape.hh" 
#include <string>
class RooTestPdf : public RooAbsPdf {
    public:
        RooTestPdf(const char *name, const char *title,
                RooAbsReal& _theta1,
                RooAbsReal& _theta2,
                RooAbsReal& _a1,
                RooAbsReal& _a2,
                bool calMCIntegral = false,
                int MCType = 0);
        RooTestPdf(const RooTestPdf& other, const char* name=0);
        virtual TObject* clone(const char* newname) const { return new RooTestPdf(*this,newname);}
        inline virtual ~RooTestPdf(){ };

        void InPutPHSPDat(TString mcdat);
        void toyMC(const Int_t events, const TString fout,
                   const Int_t sed,    const Int_t   type );

        void SetChildrenMass(const Double_t &mass1, 
                const Double_t &mass2);
        void test();
        Double_t evaluate(Double_t theta1, Double_t theta2) const;
        void readMC(TString MCFile, TString MCTree = "sig", TString Cut = "", TString Ttheta1 = "Theta1", 
                TString Ttheta2 = "Theta2", TString TWeight = "Weight");
        void readMC(TChain *MCFile, TString Cut = "", TString Ttheta1 = "Theta1", 
                TString Ttheta2 = "Theta2", TString TWeight = "Weight");
        void phsp2toy(TChain *MCFile, TString Ttheta1 = "Theta1", 
                TString Ttheta2 = "Theta2");
        RooDataSet* readData(TString DataFile, TString DataTree = "sig", TString Cut = "", RooArgSet ArgSet = RooArgSet());
        RooDataSet* readData(TChain *ch, TString Cut = "", RooArgSet ArgSet = RooArgSet());

    protected:
        RooRealProxy theta;
        RooRealProxy theta1;
        RooRealProxy theta2;
        RooRealProxy phi1;
        RooRealProxy phi2;
        RooRealProxy a1;
        RooRealProxy a2;
        RooRealProxy PL;
        RooRealProxy Rg1;
        RooRealProxy Ig1;
        RooRealProxy Rg2;
        RooRealProxy Ig2;
        RooRealProxy RFA;
        RooRealProxy IFA;
        RooRealProxy RHT;
        RooRealProxy IHT;
        //        TString TCostheta;
//        TString TCostheta1;
//        TString TCostheta2;
//        TString TPhi1;
//        TString TPhi2;

        Double_t WTTG(Double_t theta1,Double_t theta2,
                Double_t a1, Double_t a2) const;
        Double_t MCIntegral(Double_t _TTG_Itgr[98],Double_t a1, Double_t a2) const;
        double   TTG(double thetaProton, double thetaAntiProton, int i) const;
        double   paraterm(double alphaLbd, double alphaLbdbar, int i) const;
        //RooRealProxy _width;
        Double_t evaluate() const;
        bool calMCIntegral;
        int  MCType;
        TString readMCItgrFile;
        TString writeMCItgrFile;
        void readItgr(Double_t _TTG_Itgr[98], TString _file) const;
        void writeItgr(Double_t _TTG_Itgr[98], TString _file) const;
//        Double_t evaluate(Double_t _p11, Double_t _p21, Double_t );
        //_p31, Double_t _p41, Double_t _p12, Double_t _p22, Double_t
        //_p32, Double_t _p42, Double_t _p13, Double_t _p23, Double_t
        //_p33, Double_t _p43) const;

       Int_t getAnalyticalIntegral(RooArgSet& allVars, RooArgSet& analVars, const char* rangeName=0) const;
       Double_t analyticalIntegral (Int_t code, const char* rangeName) const;
        void initialize();
    private:
        RooListProxy _ParameterCol;
        TIterator *_parsItr;
        Double_t _m1, _m2;
        Double_t _rRes;
        Double_t *mcp[5], *weight;
        Double_t *TTG_Itgr;
        Double_t *my_TTG_Itgr;
        Int_t Nmc;
        Int_t _spin;
        TString _mcdat;
        TString _ToyMCItgrFile;
        TString _PHSPMCItgrFile;
        Double_t *mc_theta, *mc_phi, *mc_theta1, *mc_theta2,*mc_phi1, *mc_phi2, *mc_weight;

        ClassDef(RooTestPdf,1)
};
#endif

