#include "RooTestPdf.hh"
#include "LineShape.hh" 
#include "RooArgList.h"
#include "RooDataSet.h"
#include "RooFit.h"
#include "TTree.h"
#include "TRandom.h"
#include "TTreeFormula.h"
#include "TGenPhaseSpace.h"
//#include "varterm.cc"
//#include "facTerm.cc"
#include "TSystem.h"
#include <fstream>
#include <iostream>
#include <string>

using namespace RooFit;
using namespace std;

ClassImp(RooTestPdf)
    RooTestPdf::RooTestPdf(const char *name, const char *title,
            RooAbsReal& _theta1,
            RooAbsReal& _theta2,
            RooAbsReal& _a1,
            RooAbsReal& _a2,
            bool _calMCIntegral,
            int  _MCType):
        RooAbsPdf(name,title),
//        _ParameterCol("ParameterCol", "", this)
        theta1("theta1","theta1",this,_theta1),
        theta2("theta2","theta2",this,_theta2),
        a1("a1", "a1", this, _a1),
        a2("a2", "a2", this, _a2)
{
    Nmc = 1e15;
    MCType = _MCType;
    calMCIntegral = _calMCIntegral;
    initialize();
}
RooTestPdf::RooTestPdf(const RooTestPdf& other, const char* name):
    RooAbsPdf(other,name),
    theta1("theta1",this,other.theta1),
    theta2("theta2",this,other.theta2),
    a1("a1", this, other.a1),
    a2("a2", this, other.a2)
{
    calMCIntegral = other.calMCIntegral;
    Nmc = other.Nmc;
    MCType = other.MCType;
    weight = other.weight;
    TTG_Itgr = other.TTG_Itgr;
}

void RooTestPdf::initialize(){
    TTG_Itgr=new Double_t[50];
    for(Int_t i=0;i<50;i++){
        TTG_Itgr[i]=0;
    }
}

void RooTestPdf::readMC(TString MCFile, TString MCTree, TString Cut, 
        TString Ttheta1, TString Ttheta2, TString TWeight)
{   
    if(calMCIntegral == false) return;
    Double_t mc_theta1,mc_theta2,mc_weight;
    if(!MCFile.SubString(".dat").IsNull()){
        FILE *fp;
        if((fp=fopen(MCFile,"r"))==NULL){
            printf("can't open input file");
            return;
        }
        int i = 0;
        cout<<"read MC file from:  "<<MCFile<<endl;
        double eva = 0;
        while(fscanf(fp,"%lf%lf%lf\n",
                    &mc_theta1,&mc_theta2,&mc_weight)!=EOF)
        {
            for(Int_t j=0;j<50;j++){
                eva = TTG(mc_theta1, mc_theta2, j);
                TTG_Itgr[j] = TTG_Itgr[j] + eva * mc_weight;
            }
            i++;
        }
        fclose(fp);
        Nmc=i;
        cout<<"total events of MC is:  "<<Nmc<<endl;
        for(Int_t j=0;j<50;j++){
            std::cout<<"Before divide Nmc: TTG Itgr["<<j<<"]:\t"<<TTG_Itgr[j]<<std::endl;
        }
        for(Int_t j=0;j<50;j++){
            TTG_Itgr[j] = TTG_Itgr[j]/Nmc;
            std::cout<<"After divide Nmc: TTG Itgr["<<j<<"]:\t"<<TTG_Itgr[j]<<std::endl;
            //                cout<<"After divide Nmc: TTG Itgr["<<j<<"]:\t"<<TTG_Itgr[j]<<endl;
        }
    }
    else{
        printf("wrong input file type!");
        return;
    }
}

RooDataSet* RooTestPdf::readData(TString DataFile, TString DataTree, TString Cut, RooArgSet ArgSet)
{
    if(!DataFile.SubString(".dat").IsNull()){
        cout<<"read data file from:  "<<DataFile<<endl;
        return RooDataSet::read(DataFile, ArgSet);
    }
    else{
        printf("wrong input file type!");
        exit;
//        return *RooDataSet::RooDataSet();
    }
}

Double_t RooTestPdf::evaluate() const
{
   double pdf  = WTTG(theta1,theta2,a1,a2);
   if(pdf<0) {
       cout<<"negtive value: "<<pdf<<endl;
       pdf = 1e-10;
   }
   return pdf;
}
Double_t RooTestPdf::evaluate(Double_t m_theta1,Double_t m_theta2) const
{
   double pdf =  WTTG(m_theta1,m_theta2,a1,a2);  
   if(pdf<0) {
       cout<<"negtive value: "<<pdf<<endl;
       pdf = 1e-10;
   }
   return pdf;
}

Int_t RooTestPdf::getAnalyticalIntegral(RooArgSet& allVars, RooArgSet& analVars, const char* rangeName)const {
    RooArgSet theSet(theta1.arg(),theta2.arg());
    if (matchArgs(allVars, analVars, theSet)){
        return 1; 
    } 
    return 0; 
}

Double_t RooTestPdf::analyticalIntegral (Int_t code, const char* rangeName)const{
    assert (code == 1);
    Double_t sum=0;
    sum = MCIntegral(TTG_Itgr,a1,a2);
    return sum;
}

void RooTestPdf::toyMC(const Int_t events, const TString fout,
                      const Int_t sed,    const Int_t type )
{
    cout<<"begin generating toy MC..."<<endl;
    if (type == 0)
        cout<<"total "<<events<<" events, "<<"type: toy PHSP"<<endl;
    else {
        cout<<"total "<<events<<" events, "<<"type: toy data"<<endl;
        cout<<"Initial Parameters:"<<endl;
        cout<<"a1:\t"<<a1<<endl;
        cout<<"a2:\t"<<a2<<endl;
    }
    Double_t Theta1,Theta2,toy_theta1,toy_theta2;
    TRandom3 random;
    random.SetSeed(sed);
    ofstream outdat(fout);
    outdat.precision(8);
    outdat.setf(ios::fixed);
    Double_t max = 1e-10;
    gRandom = new TRandom3(0);
    for(int ii =0; ii< 1e4; ii++)
    {
        toy_theta1 = random.Uniform(0, 1);
        toy_theta2 = random.Uniform(0, 1);
        Double_t amp = evaluate(toy_theta1,toy_theta2);
//        cout<<"amp: "<<amp<<endl;
        if(amp > max){
            max = amp;
            cout<<ii<<"'s events  "<<"max: "<<max<<endl;
        }
    }
    int n=0;
    while(1){

        toy_theta1 = random.Uniform(0, 1);
        toy_theta2 = random.Uniform(0, 1);
        
        Double_t amp = evaluate(toy_theta1,toy_theta2);
        
        if(type!=0){
            if(random.Uniform(0, max*1.2) >amp) continue;
        }
        Double_t toy_weight = 1.000;
        outdat<<toy_theta1<<" ";
        outdat<<toy_theta2<<" ";
        outdat<<toy_weight<<endl;            
        n++;
//        cout<<"events "<<n<<": amps = "<<amp<<endl;
        if(n>=events) break;  
    }
    outdat.close();
    cout<<"finalizing generate toy MC..."<<endl;
}

Double_t RooTestPdf::WTTG(Double_t theta1,Double_t theta2,Double_t a1, Double_t a2) const
{
  Double_t vv=0;
  for(int i=0;i<50;i++){
    vv += paraterm(a1,a2,i)*TTG(theta1,theta2,i);
  }
  return vv;
}

Double_t RooTestPdf::MCIntegral(Double_t _TTG_Itgr[50],Double_t a1, Double_t a2) const
{
  // Specify complete amplitude squared
  Double_t vv=0;
  for(int i=0;i<50;i++){
    vv += paraterm(a1,a2,i)*_TTG_Itgr[i];
  }
  return vv;
}
double RooTestPdf::paraterm(double a1, double a2, int i) const{ 

    double val = 0;

    switch (i){
        case 0:
            val = a1;
            break;
        case 1:
            val = a2;
            break;
        default:
            val = 0;
    }
    return val;
}
double  RooTestPdf::TTG(double theta1, double theta2, int i) const{
    double val = 0;
    switch (i){
        case 0:
            val = theta1*theta1;
            break;
        case 1:
            val = theta2;
            break;
        default:
            val = 0;
    }
    return val;
}
